@@ -484,6 +484,15 @@ class ErrorCodes(object):
         HasAlreadyBeenRefunded = "91512"
         IdealPaymentNotComplete = "815141"
         IdealPaymentsCannotBeVaulted = "915150"
+        DiscountAmountFormatIsInvalid = "915159"
+        DiscountAmountCannotBeNegative = "915160"
+        DiscountAmountIsTooLarge = "915161"
+        ShippingAmountFormatIsInvalid = "915162"
+        ShippingAmountCannotBeNegative = "915163"
+        ShippingAmountIsTooLarge = "915164"
+        ShipsFromPostalCodeIsTooLong = "915165"
+        ShipsFromPostalCodeIsInvalid = "915166"
+        ShipsFromPostalCodeInvalidCharacters = "915167"
         MerchantAccountDoesNotMatch3DSecureMerchantAccount = "91584"
         MerchantAccountDoesNotMatchIdealPaymentMerchantAccount = "915143"
         MerchantAccountDoesNotSupportMOTO = "91558"
@@ -547,6 +556,8 @@ class ErrorCodes(object):
         ThreeDSecureXidIsRequired = "915115"
         ThreeDSecureEciFlagIsInvalid = "915114"
         ThreeDSecureMerchantAccountDoesNotSupportCardType = "915131"
+        TooManyLineItems = "915157"
+        LineItemsExpected = "915158"
         TypeIsInvalid = "91523"
         TypeIsRequired = "91524"
         UnsupportedVoiceAuthorization = "91539"
@@ -577,3 +588,30 @@ class ErrorCodes(object):
                 DepartureDateIsInvalid = "93411"
                 LodgingCheckInDateIsInvalid = "93412"
                 LodgingCheckOutDateIsInvalid = "93413"
+
+        class LineItem(object):
+            CommodityCodeIsTooLong = "95801"
+            DescriptionIsTooLong = "95803"
+            DiscountAmountFormatIsInvalid = "95804"
+            DiscountAmountIsTooLarge = "95805"
+            DiscountAmountMustBeGreaterThanZero = "95806"
+            KindIsInvalid = "95807"
+            KindIsRequired = "95808"
+            NameIsRequired = "95822"
+            NameIsTooLong = "95823"
+            ProductCodeIsTooLong = "95809"
+            QuantityFormatIsInvalid = "95810"
+            QuantityIsRequired = "95811"
+            QuantityIsTooLarge = "95812"
+            TotalAmountFormatIsInvalid = "95813"
+            TotalAmountIsRequired = "95814"
+            TotalAmountIsTooLarge = "95815"
+            TotalAmountMustBeGreaterThanZero = "95816"
+            UnitAmountFormatIsInvalid = "95817"
+            UnitAmountIsRequired = "95818"
+            UnitAmountIsTooLarge = "95819"
+            UnitAmountMustBeGreaterThanZero = "95820"
+            UnitOfMeasureIsTooLarge = "95821"
+            UnitTaxAmountFormatIsInvalid = "95824"
+            UnitTaxAmountIsTooLarge = "95825"
+            UnitTaxAmountMustBeGreaterThanZero = "95826"
