@@ -3,6 +3,7 @@ import warnings
 from decimal import Decimal
 from braintree.add_on import AddOn
 from braintree.apple_pay_card import ApplePayCard
+from braintree.authorization_adjustment import AuthorizationAdjustment
 from braintree.coinbase_account import CoinbaseAccount
 from braintree.android_pay_card import AndroidPayCard
 from braintree.amex_express_checkout_card import AmexExpressCheckoutCard
@@ -27,7 +28,12 @@ from braintree.exceptions.not_found_error import NotFoundError
 from braintree.descriptor import Descriptor
 from braintree.risk_data import RiskData
 from braintree.three_d_secure_info import ThreeDSecureInfo
+from braintree.transaction_line_item import TransactionLineItem
 from braintree.us_bank_account import UsBankAccount
+from braintree.ideal_payment import IdealPayment
+from braintree.visa_checkout_card import VisaCheckoutCard
+from braintree.masterpass_card import MasterpassCard
+from braintree.facilitated_details import FacilitatedDetails
 from braintree.facilitator_details import FacilitatorDetails
 from braintree.payment_instrument_type import PaymentInstrumentType
 
@@ -91,6 +97,7 @@ class Transaction(Resource):
         "id",
         "additional_processor_response",
         "amount",
+        "authorization_adjustments",
         "avs_error_response_code",
         "avs_postal_code_response_code",
         "avs_street_address_response_code",
@@ -100,6 +107,7 @@ class Transaction(Resource):
         "currency_iso_code",
         "customer_id",
         "cvv_response_code",
+        "discount_amount",
         "disputes",
         "escrow_status",
         "gateway_rejection_reason",
@@ -120,6 +128,8 @@ class Transaction(Resource):
         "refunded_transaction_id",
         "service_fee_amount",
         "settlement_batch_id",
+        "shipping_amount",
+        "ships_from_postal_code",
         "status",
         "status_history",
         "sub_merchant_account_id",
@@ -307,6 +317,13 @@ class Transaction(Resource):
         """
         return Configuration.gateway().transaction.find(transaction_id)
 
+    @staticmethod
+    def line_items(transaction_id):
+        """
+        Find a transaction's line items, given a transaction_id. This does not return
+        a result object. This will raise a :class:`NotFoundError <braintree.exceptions.not_found_error.NotFoundError>` if the provided transaction_id is not found. ::
+        """
+        return Configuration.gateway().transaction_line_item.find_all(transaction_id)
 
     @staticmethod
     def hold_in_escrow(transaction_id):
@@ -488,7 +505,8 @@ class Transaction(Resource):
             "amount", "customer_id", "device_session_id", "fraud_merchant_id", "merchant_account_id", "order_id", "channel",
             "payment_method_token", "purchase_order_number", "recurring", "transaction_source", "shipping_address_id",
             "device_data", "billing_address_id", "payment_method_nonce", "tax_amount",
-            "shared_payment_method_token", "shared_customer_id", "shared_billing_address_id", "shared_shipping_address_id",
+            "shared_payment_method_token", "shared_customer_id", "shared_billing_address_id", "shared_shipping_address_id", "shared_payment_method_nonce",
+            "discount_amount", "shipping_amount", "ships_from_postal_code",
             "tax_exempt", "three_d_secure_token", "type", "venmo_sdk_payment_method_code", "service_fee_amount",
             {
                 "risk_data": [
@@ -537,6 +555,8 @@ class Transaction(Resource):
                     "venmo_sdk_session",
                     "payee_email",
                     "skip_advanced_fraud_checking",
+                    "skip_avs",
+                    "skip_cvv",
                     {
                         "paypal": [
                             "payee_email",
@@ -556,7 +576,8 @@ class Transaction(Resource):
                         "venmo_merchant_data": [
                             "venmo_merchant_public_id",
                             "originating_transaction_id",
-                            "originating_merchant_id"
+                            "originating_merchant_id",
+                            "originating_merchant_kind"
                         ],
                     },
                     {
@@ -579,8 +600,13 @@ class Transaction(Resource):
                             ]
                         }
                     ]
-                }
-            ]
+                },
+            {"line_items":
+                [
+                    "quantity", "name", "description", "kind", "unit_amount", "unit_tax_amount", "total_amount", "discount_amount", "unit_of_measure", "product_code", "commodity_code", "url",
+                ]
+            },
+        ]
 
     @staticmethod
     def submit_for_settlement_signature():
@@ -619,6 +645,10 @@ class Transaction(Resource):
         self.amount = Decimal(self.amount)
         if self.tax_amount:
             self.tax_amount = Decimal(self.tax_amount)
+        if "discount_amount" in attributes and self.discount_amount:
+            self.discount_amount = Decimal(self.discount_amount)
+        if "shipping_amount" in attributes and self.shipping_amount:
+            self.shipping_amount = Decimal(self.shipping_amount)
         if "billing" in attributes:
             self.billing_details = Address(gateway, attributes.pop("billing"))
         if "credit_card" in attributes:
@@ -629,6 +659,8 @@ class Transaction(Resource):
             self.europe_bank_account_details = EuropeBankAccount(gateway, attributes.pop("europe_bank_account"))
         if "us_bank_account" in attributes:
             self.us_bank_account = UsBankAccount(gateway, attributes.pop("us_bank_account"))
+        if "ideal_payment" in attributes:
+            self.ideal_payment_details = IdealPayment(gateway, attributes.pop("ideal_payment"))
         if "apple_pay" in attributes:
             self.apple_pay_details = ApplePayCard(gateway, attributes.pop("apple_pay"))
         if "coinbase_account" in attributes:
@@ -639,6 +671,10 @@ class Transaction(Resource):
             self.amex_express_checkout_card_details = AmexExpressCheckoutCard(gateway, attributes.pop("amex_express_checkout_card"))
         if "venmo_account" in attributes:
             self.venmo_account_details = VenmoAccount(gateway, attributes.pop("venmo_account"))
+        if "visa_checkout_card" in attributes:
+            self.visa_checkout_card_details = VisaCheckoutCard(gateway, attributes.pop("visa_checkout_card"))
+        if "masterpass_card" in attributes:
+            self.masterpass_card_details = MasterpassCard(gateway, attributes.pop("masterpass_card"))
         if "customer" in attributes:
             self.customer_details = Customer(gateway, attributes.pop("customer"))
         if "shipping" in attributes:
@@ -657,6 +693,8 @@ class Transaction(Resource):
             self.disbursement_details = DisbursementDetail(attributes.pop("disbursement_details"))
         if "disputes" in attributes:
             self.disputes = [Dispute(dispute) for dispute in self.disputes]
+        if "authorization_adjustments" in attributes:
+            self.authorization_adjustments = [AuthorizationAdjustment(authorization_adjustment) for authorization_adjustment in self.authorization_adjustments]
         if "payment_instrument_type" in attributes:
             self.payment_instrument_type = attributes["payment_instrument_type"]
 
@@ -668,6 +706,8 @@ class Transaction(Resource):
             self.three_d_secure_info = ThreeDSecureInfo(attributes["three_d_secure_info"])
         else:
             self.three_d_secure_info = None
+        if "facilitated_details" in attributes:
+            self.facilitated_details = FacilitatedDetails(attributes.pop("facilitated_details"))
         if "facilitator_details" in attributes:
             self.facilitator_details = FacilitatorDetails(attributes.pop("facilitator_details"))
 
@@ -706,3 +746,9 @@ class Transaction(Resource):
     def is_disbursed(self):
        return self.disbursement_details.is_valid
 
+    @property
+    def line_items(self):
+        """
+        The line items associated with this transaction
+        """
+        return self.gateway.transaction_line_item.find_all(self.id)
